import AsyncHelper from "./tool/AsyncHelper";

const _global = (window /* browser */ || global /* node */) as any;

// 绑定一个函数闭包
_global.bindFunction = function (target: any, func: any, ...args: any[]): any {
    if (func == null) {
        cc.log("function is null");
        return null;
    }
    return function () {
        return func.call(target, ...args);
    };
};

_global.bindDelegate = function (target: any, func: any): any {
    if (func == null) {
        cc.log("function is null");
        return null;
    }
    return function (...args) {
        return func.call(target, ...args);
    };
};

/**
 * 替换所有匹配exp的字符串为指定字符串
 * @param exp 被替换部分的正则
 * @param newStr 替换成的字符串
 */
_global.replaceAll = function (str: string, exp: string, newStr: string): string {
    return str.replace(new RegExp(exp, "gm"), newStr);
};

/**
 * 原型：字符串格式化
 * @param args 格式化参数值
 */
_global.format = function (str: string, ...args: any[]): string {
    let result = str;
    if (args.length < 1) {
        return result;
    }

    for (const key of Object.keys(args)) {
        const value = args[key];
        if (undefined != value) {
            result = replaceAll(result, "\\{" + key + "\\}", value);
        }
    }
    return result;
};

_global.remove = function (array: any[], element: any): void {
    for (let i = 0; i < array.length; i++) {
        if (array[i] === element) {
            array.splice(i, 1);
            return;
        }
    }
};

_global.findAll = function (array: any[], predicate: (item: any) => boolean): any[] {
    const arr = new Array();
    for (const item of array) {
        if (predicate(item)) {
            arr.push(item);
        }
    }
    return arr;
};

_global.findLast = function (array: any[], predicate: (item: any) => boolean): any {
    for (let index = array.length - 1; index >= 0; index--) {
        const element = array[index];
        if (predicate(element)) {
            return element;
        }
    }
    return null;
};

_global.Choose = function (condition: boolean, ifTrue: any, ifFalse: any): any { return condition ? ifTrue : ifFalse; };
_global.SafeCall = function (targetOrFunc: any, funcOrArg: any = null, ...args: any[]): any {
    if (targetOrFunc == null && funcOrArg == null) {
        return null;
    }
    if (targetOrFunc != null && typeof (targetOrFunc) == "function") {
        return targetOrFunc.call(window, funcOrArg, ...args);
    } else if (funcOrArg != null && typeof (funcOrArg) == "function") {
        return funcOrArg.call(targetOrFunc, ...args);
    } else {
        cc.error(arguments);
        return null;
    }
};

_global.CopyArr = (src: any[], dst: any[]): void => {
    for (let item of src) {
        dst.push(item);
    }
};

_global.CheckClickIsOnAGameObject = function (): boolean {
    // if (CommonUtil.IsIPhone() || CommonUtil.IsAndroid() ) {
    //     if (Input.touchCount <=0 ) {
    //         return true
    //     }
    //     return EventSystem.current.IsPointerOverGameObject(Input.GetTouch(0).fingerId)
    // } else {
    //     return EventSystem.current.IsPointerOverGameObject()
    // }
    return false;
};

_global.AsyncCall = function (func: (...args: any[]) => void, delay: number = 0, node: cc.Node = null, target: any = null, ...args: any[]): NodeJS.Timeout {
    return AsyncHelper.GetInstance(node).waitForSeconds(delay, func, target, ...args);
};

_global.StopAsyncCall = function (node: cc.Node, timeoutId: NodeJS.Timeout): void {
    AsyncHelper.GetInstance(node).stopAsync(timeoutId);
};

_global.StopAllAsyncCall = function (node: cc.Node): void {
    AsyncHelper.GetInstance(node).stopAllAsync();
};
