
const configDirName = "config";
const sysConfigDirName = "sysConfig";

export default class ConfigManager {

    static Init(callback: () => void) {
        if (ConfigManager._init == true) {
            return;
        }
        ConfigManager.LoadConfig(function () {
            ConfigManager._init = true;
            cc.log("Load Config Finished");
            callback();
        });
    }

    static LoadConfig(callback) {
        cc.loader.loadResDir(configDirName, function (error, jsonAsset) {
            if (error != null) {
                if (ConfigManager.retryCount < ConfigManager.maxRetryCount) {
                    ConfigManager.LoadConfig(callback);
                    ConfigManager.retryCount++;
                }
            } else {
                {
                    // self.retryCount = 0;
                    // CommonUtil.showDialogUI("", function(){
                    // self.loadConfig(configDirName, callback)
                    // })
                }
            }
            ConfigManager.retryCount = 0;
            for (const iterator of jsonAsset) {
                ConfigManager.configs[iterator.name] = iterator.json;
            }
            ConfigManager.LoadSysConfig(callback);
        });
    }

    static LoadSysConfig(callback) {
        cc.loader.loadResDir(sysConfigDirName, function (error, jsonAsset) {
            if (error != null) {
                if (ConfigManager.retryCount < ConfigManager.maxRetryCount) {
                    ConfigManager.LoadConfig(callback);
                    ConfigManager.retryCount++;
                }
            } else {
                {
                    // self.retryCount = 0;
                    // CommonUtil.showDialogUI("", function(){
                    // self.loadConfig(configDirName, callback)
                    // })
                }
            }
            ConfigManager.retryCount = 0;
            for (const iterator of jsonAsset) {
                ConfigManager.sysConfigs[iterator.name] = iterator.json;
            }
            SafeCall(callback);
        });
    }

    static GetSpritePath(key: string): string {
        let spriteIndex = this.GetSysConfig("SpriteIndex");
        if (!spriteIndex.hasOwnProperty(key)) {
            cc.warn("!!!no sprite key :" + key);
            return "";
        }
        return spriteIndex[key];
    }

    static GetPrefabPath(key: string): string {
        let prefabIndex = this.GetSysConfig("PrefabIndex");
        if (!prefabIndex.hasOwnProperty(key)) {
            cc.warn("!!!no prefab key :" + key);
            return "";
        }
        return prefabIndex[key];
    }

    static GetRegion(): string {
        return ConfigManager.region;
    }

    static Clear(): void {
        ConfigManager.configs = {};
        ConfigManager.sysConfigs = {};
        ConfigManager._init = false;
    }

    static GetConfig(name: string): any {
        let config = this.configs[name];

        if (config == null) {
            cc.warn("Config not found:", name);
        }
        return config;
    }

    static GetSysConfig(name: string): any {
        let config = this.sysConfigs[name];
        if (config == null) {
            cc.warn("Config not found:", name);
        }
        return config;
    }

    static GetText(key: string, ...args): string {
        let textConfig = this.GetConfig("language");
        let text = textConfig[key];

        if (text == null) {
            cc.warn("不存在 [" + key + "] 字符串");
            return key;
        } else {
            return format(text[this.region], ...args);
        }
    }

    static GetNameText(key: string): string {
        if (key == null) {
            cc.error("text key is nil");
            return "None";
        }
        return this.GetText("name_" + key);
    }

    static GetDescText(key: string): string {
        if (key == null) {
            cc.error("text key is nil", key);
            return "None";
        }
        return this.GetText("desc_" + key);
    }

    static GetFormatText(key: string, ...args): string {
        let text = this.GetText(key, ...args);
        if (text != null) {
            return text; // format(text, ...args);
        }
        return key;
    }

    static GetConst(key: string): any {
        let value = this.GetConfig("Cat").cat[key];
        if (value != null) {
            return value;
        } else {
            cc.warn("no key[ ", key, " ] in Cat table");
        }
    }

    static GetCommon(key: string): any {
        let value = this.GetConfig("Common").common[key];
        if (value != null) {
            return value;
        } else {
            cc.warn("no key[ ", key, " ] in Common table");
        }
    }

    static GetPlatform(): string {
        return this.GetCommon("platform");
    }

    private static _init: boolean = false;
    private static region: string;

    private static configs = {};
    private static sysConfigs = {};
    private static retryCount: number = 0;
    private static maxRetryCount: number = 0;
}
