import ConfigManager from "./ConfigManager";

const FRAME_RATE: number = 30;
const NET_KEEP_ALIVE_TIME: number = 3000;

const { ccclass } = cc._decorator;
@ccclass
export class GameManager extends cc.Component {

    // 创建单例Node
    static getInstance(): GameManager {
        if (GameManager._instance == null) {
            let node = new cc.Node("GameManager");
            GameManager._instance = node.addComponent("GameManager");
            cc.game.addPersistRootNode(node);
        }
        return GameManager._instance;
    }

    private static _instance: GameManager = null;

    offLineTime: number;

    isLogin = false;

    private _leaveTimerId = null;

    private _initialized: boolean = false;

    private energyTime: number;

    private loadADTime: number;
    private storageTimer: number;
    private storageTime: number;
    private needStorage: boolean;
    private energyRecoverTime: number;
    private maxEnergy: number;
    private root: any;

    private startUpdate: boolean = false;

    onLoad(): void {
        cc.game.setFrameRate(FRAME_RATE);

        const target = this;
        cc.game.on(cc.game.EVENT_HIDE, function () {
            target._leaveTimerId = AsyncCall(() => {
                cc.log("Close connection onHide");
                target._leaveTimerId = -1;
            }, NET_KEEP_ALIVE_TIME);
        });
        cc.game.on(cc.game.EVENT_SHOW, function () {
            if (target._leaveTimerId != -1) {
                StopAsyncCall(null, target._leaveTimerId);
            }

        });
    }

    init(obj: any, callback: () => void): void {
        if (this._initialized) {
            SafeCall(obj, callback);
            return;
        }
        this._initialized = true;
        cc.log("GameManager init");
        cc.director.getPhysicsManager().enabled = true; // 碰撞检测
        cc.director.getCollisionManager().enabled = true; // 物理引擎
        // cc.director.getPhysicsManager().debugDrawFlags = cc.PhysicsManager.DrawBits.e_aabbBit |
        //     cc.PhysicsManager.DrawBits.e_jointBit |
        //     cc.PhysicsManager.DrawBits.e_shapeBit;

        const target = this;
        ConfigManager.Init(function () {
            SafeCall(obj, callback);
        });

    }

    onDestroy() {
        this.isLogin = false;
        this.offLineTime = 0;
        GameManager._instance = null;
    }

    getSave(): void {
    }

    update(dt: number): void {

    }

}
