
const { ccclass, property } = cc._decorator;
const MIN_DELAY: number = 0.0001;

@ccclass
export default class ScheduleQueue extends cc.Component {

    @property
    _queue = null;

    @property
    _running = false;

    onLoad() {
        this._queue = new Array();
    }
    add(component, func, delay) {
        const data: any = {};
        data.component = component;
        data.func = func;
        data.delay = delay;
        this._queue.push(data);
        this.scheduleNext();
    }

    scheduleNext() {
        if (this._running) {
            return;
        }

        const data = this._queue.shift();
        if (data != null) {
            this._running = true;
            // GameGlobal.scene.blockInput(true);

            const self = this;
            if (data.delay > MIN_DELAY) {
                data.component.scheduleOnce(function () {
                    try {
                        data.func.call(data.component);
                    } catch (error) {
                        cc.error(error);
                    }
                    self._running = false;
                    // GameGlobal.scene.blockInput(false);
                    self.scheduleNext();
                }, data.delay);
            } else {
                try {
                    data.func.call(data.component);
                } catch (error) {
                    cc.error(error);
                }
                self._running = false;
                self.scheduleNext();
            }
        }
    }
}
