import CommonUtil from "../util/CommonUtil";

const { ccclass, property } = cc._decorator;

@ccclass
export default class ButtonControl extends cc.Component {
    @property(cc.Sprite)
    background: cc.Sprite = null;
    @property(cc.SpriteFrame)
    backgroundInteractableImg: cc.SpriteFrame = null;
    @property(cc.SpriteFrame)
    backgroundUninteractableImg: cc.SpriteFrame = null;
    @property(cc.Node)
    setColorTarget: cc.Node[] = [];
    @property(cc.Color)
    interactableColor: cc.Color = null;
    @property(cc.Color)
    uninteractableColor: cc.Color = null;
    @property
    needSetColor = true;

    @property
    _interactable = true;
    @property
    _btn: cc.Button = null;
    @property
    _btnEffect: cc.Node = null;

    onLoad() {
        this._btn = this.node.getComponent(cc.Button);
        this._btnEffect = CommonUtil.findAChild(this.node, "ButtonEffect");
        this.setInteractable();
    }

    update(dt) {
        if (this._btn == null) {
            return;
        }
        if (this._interactable != this._btn.interactable) {
            this.setInteractable();
        }
    }

    setInteractable() {
        this._interactable = this._btn.interactable;
        if (this.backgroundInteractableImg != null && this.backgroundUninteractableImg != null) {
            this.background.spriteFrame = this._interactable ? this.backgroundInteractableImg : this.backgroundUninteractableImg;
            CommonUtil.SetActive(this._btnEffect, "", this._interactable);
        }
        if (this.needSetColor) {
            this.setColor();
        }
    }
    setColor() {
        for (const iterator of this.setColorTarget) {
            iterator.color = this._interactable ? this.interactableColor : this.uninteractableColor;
        }
    }
}
