const { ccclass, property } = cc._decorator;
@ccclass
export default class CCDelegate extends cc.Component {

    private _onUpdate: (_dt: number) => void = null;
    private _onTouchStart: (event: cc.Touch) => void = null;
    private _onTouchEnd: (event: cc.Touch) => void = null;
    private _onTouchMove: (event: cc.Touch) => void = null;
    private _onTouchCancel: (event: cc.Touch) => void = null;

    // 此处必须开启 cc.director.getCollisionManager().enabled = true;
    private _onCollisionEnter: (other: cc.Collider, self: cc.Collider) => void = null;
    private _onCollisionStay: (other: cc.Collider, self: cc.Collider) => void = null;
    private _onCollisionLeave: (other: cc.Collider, self: cc.Collider) => void = null;

    /**
     * 此处必须开启 rigidBody.enabledContactListener = true;
     * contact 中比较常用的信息就是碰撞的位置和法向量，contact 内部是按照刚体的本地坐标来存储信息的
     * ，而我们一般需要的是世界坐标系下的信息，我们可以通过 contact.getWorldManifold 来获取这些信息。
     */
    private _onBeginContact: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void = null;
    private _onEndContact: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void = null;
    private _onPreSolve: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void = null;
    private _onPostSolve: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void = null;

    onLoad(): void {
        let node: any = this.node;
        node.on(cc.Node.EventType.TOUCH_START, this.onTouchStart, this);
        node.on(cc.Node.EventType.TOUCH_END, this.onTouchEnd, this);
        node.on(cc.Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
        node.on(cc.Node.EventType.TOUCH_CANCEL, this.onTouchCancel, this);
        // node._hitTest = this.hitTest.bind(this);
    }

    hitTest(point: cc.Vec2): boolean {
        return true;
    }

    update(dt: number): void {
        if (this._onUpdate != null) {
            this._onUpdate(dt);
        }
    }

    registerOnUpdate(func: (_dt: number) => void): void {
        this._onUpdate = func;
    }

    registerOnTouchStart(func: (event: cc.Touch) => void): void {
        this._onTouchStart = func;
    }

    registerOnTouchEnd(func: (event: cc.Touch) => void): void {
        this._onTouchEnd = func;
    }

    registerOnTouchMove(func: (event: cc.Touch) => void): void {
        this._onTouchMove = func;
    }

    registerOnTouchCancel(func: (event: cc.Touch) => void): void {
        this._onTouchCancel = func;
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     */
    registerOnCollisionEnter(func: (other: cc.Collider, self: cc.Collider) => void): void {
        this._onCollisionEnter = func;
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     */
    registerOnCollisionStay(func: (other: cc.Collider, self: cc.Collider) => void): void {
        this._onCollisionStay = func;
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     */
    registerOnCollisionLeave(func: (other: cc.Collider, self: cc.Collider) => void): void {
        this._onCollisionLeave = func;
    }

    /**
     * 此处必须开启 rigidBody.enabledContactListener = true;
     * 只在两个碰撞体开始接触时被调用一次
     * @param func
     */
    registerOnBeginContact(func: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void): void {
        this._onBeginContact = func;
    }

    /**
     * 此处必须开启 rigidBody.enabledContactListener = true;
     * 只在两个碰撞体结束接触时被调用一次
     * @param func
     */
    registerOnEndContact(func: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void): void {
        this._onEndContact = func;
    }

    /**
     * 此处必须开启 rigidBody.enabledContactListener = true;
     * 每次将要处理碰撞体接触逻辑时被调用
     * @param func
     */
    registerOnPreSolve(func: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void): void {
        this._onPreSolve = func;
    }

    /**
     * 此处必须开启 rigidBody.enabledContactListener = true;
     * 每次处理完碰撞体接触逻辑时被调用
     * @param func
     */
    registerOnPostSolve(func: (contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider) => void): void {
        this._onPostSolve = func;
    }

    onTouchStart(event: cc.Touch): void {
        // cc.log("onTouchStart: ", event.getLocation());
        if (this._onTouchStart != null) {
            this._onTouchStart(event);
        }
    }

    onTouchEnd(event: cc.Touch): void {
        // cc.log("onTouchEnd: ", event.getLocation());
        if (this._onTouchEnd != null) {
            this._onTouchEnd(event);
        }
    }

    onTouchMove(event: cc.Touch): void {
        // cc.log("onTouchMove: ", event.getLocation());
        if (this._onTouchMove != null) {
            this._onTouchMove(event);
        }
    }

    onTouchCancel(event: cc.Touch): void {
        // cc.log("onTouchCancel: ", event.getLocation());
        if (this._onTouchCancel != null) {
            this._onTouchCancel(event);
        }
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     * 当碰撞产生的时候调用
     * @param  {Collider} other 产生碰撞的另一个碰撞组件
     * @param  {Collider} self  产生碰撞的自身的碰撞组件
     */
    onCollisionEnter(other: cc.Collider, self: cc.Collider): void {

        // cc.log("on collision enter", self, other);
        if (this._onCollisionEnter != null) {
            this._onCollisionEnter(other, self);
        }

        // // 碰撞系统会计算出碰撞组件在世界坐标系下的相关的值，并放到 world 这个属性里面
        // let world = self.world;

        // // 碰撞组件的 aabb 碰撞框
        // let aabb = world.aabb;

        // // 节点碰撞前上一帧 aabb 碰撞框的位置
        // let preAabb = world.preAabb;

        // // 碰撞框的世界矩阵
        // let t = world.transform;

        // // 以下属性为圆形碰撞组件特有属性
        // let r = world.radius;
        // let p = world.position;

        // // 以下属性为 矩形 和 多边形 碰撞组件特有属性
        // let ps = world.points;
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     * 当碰撞产生后，碰撞结束前的情况下，每次计算碰撞结果后调用
     * @param  {Collider} other 产生碰撞的另一个碰撞组件
     * @param  {Collider} self  产生碰撞的自身的碰撞组件
     */
    onCollisionStay(other: cc.Collider, self: cc.Collider): void {
        if (this._onCollisionStay != null) {
            this._onCollisionStay(other, self);
        }
    }

    /**
     * 此处必须开启 cc.director.getCollisionManager().enabled = true;
     * 当碰撞结束后调用
     * @param  {Collider} other 产生碰撞的另一个碰撞组件
     * @param  {Collider} self  产生碰撞的自身的碰撞组件
     */
    onCollisionExit(other: cc.Collider, self: cc.Collider): void {
        if (this._onCollisionLeave != null) {
            this._onCollisionLeave(other, self);
        }
    }

    /**
     * 只在两个碰撞体开始接触时被调用一次
     * @param contact
     * @param selfCollider
     * @param otherCollider
     */
    onBeginContact(contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider): void {
        if (this._onBeginContact) {
            this._onBeginContact(contact, selfCollider, otherCollider);
        }
    }

    /**
     * 只在两个碰撞体结束接触时被调用一次
     * @param contact
     * @param selfCollider
     * @param otherCollider
     */
    onEndContact(contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider): void {
        if (this._onEndContact) {
            this._onEndContact(contact, selfCollider, otherCollider);
        }
    }

    /**
     * 每次将要处理碰撞体接触逻辑时被调用
     * @param contact
     * @param selfCollider
     * @param otherCollider
     */
    onPreSolve(contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider): void {
        if (this._onPreSolve) {
            this._onPreSolve(contact, selfCollider, otherCollider);
        }
    }

    /**
     * 每次处理完碰撞体接触逻辑时被调用
     * @param contact
     * @param selfCollider
     * @param otherCollider
     */
    onPostSolve(contact: cc.PhysicsContact, selfCollider: cc.Collider, otherCollider: cc.Collider): void {
        if (this._onPostSolve) {
            this._onPostSolve(contact, selfCollider, otherCollider);
        }
    }

}
