import UIManager from "../ui/uiFrame/UIManager";
import CommonUtil from "../util/CommonUtil";

const { ccclass, property } = cc._decorator;

const HOLD_MAX_TIME: number = 400;

@ccclass
export default class TouchScrollUI extends cc.Component {

    // @property(cc.Node)
    // areas: cc.Node[] = [];
    @property
    offsetX: number = 100;
    @property(cc.Node)
    scrollItem: cc.Node = null;
    @property(cc.Button)
    leftBtn: cc.Button = null;
    @property(cc.Button)
    rightBtn: cc.Button = null;
    @property
    _itemSize: cc.Vec2 = null;
    @property
    _startPos: cc.Vec2 = null;
    @property
    _beginPos: cc.Vec2 = null;
    @property
    _currentPos: cc.Vec2 = null;
    @property
    _itemCount = 3;
    @property
    _currentItem = 1;  // index从0开始
    @property
    _finalIndex = 0;
    @property
    _resetCallBack = null;
    @property
    _moveCallBack = null;
    @property
    _isMove: boolean = false;
    @property
    _canTouchMove: boolean = true;
    @property
    _movePos: number = 0;
    @property
    _isTouch: boolean = false;
    @property
    _currentTween: cc.Tween = null;

    private lastUpdateTime: number;

    init(itemCount: number, currentItem: number, resetCallBack = null, moveCallBack = null) {
        if (this._currentTween != null) {
            this._currentTween.stop();
        }
        if (itemCount <= 0) {
            return;
        }
        if (currentItem < 0) {
            currentItem = 0;
        }
        if (currentItem >= itemCount) {
            currentItem = itemCount - 1;
        }
        this._itemCount = itemCount;
        this._currentItem = currentItem;
        this._finalIndex = currentItem;
        this._resetCallBack = resetCallBack;
        this._moveCallBack = moveCallBack;
        this._itemSize = cc.v2(this.node.width, this.node.height);
        this.scrollItem.width = this._itemSize.x;
        this.scrollItem.height = this._itemSize.y;
        let count = this._itemCount;
        if (count > 3) {
            count = 3;
        }
        this.node.width = this._itemSize.x * count;

        this.createItem();

        this.node.on(cc.Node.EventType.TOUCH_START, this.touchStart, this, true);
        this.node.on(cc.Node.EventType.TOUCH_MOVE, this.touchMove, this, true);
        this.node.on(cc.Node.EventType.TOUCH_END, this.touchEnd, this, true);
        CommonUtil.setButtonClick(this.leftBtn.node, null, this.node, cc.js.getClassName(this), "onClickLeftBtn", null);
        CommonUtil.setButtonClick(this.rightBtn.node, null, this.node, cc.js.getClassName(this), "onClickRightBtn", null);
    }

    reset(itemCount: number, currentItem: number, resetCallBack = null, moveCallBack = null) {
        if (this._currentTween != null) {
            this._currentTween.stop();
        }
        this._itemCount = itemCount;
        this._currentItem = currentItem;
        this._finalIndex = currentItem;
        this._resetCallBack = resetCallBack;
        this._moveCallBack = moveCallBack;
        this._isMove = false;
        this.createItem();
    }

    isFirst(index: number): boolean {
        return index <= 0;
    }

    isFinal(index: number): boolean {
        return this._itemCount - index <= 1;
    }

    createItem(): void {
        CommonUtil.removeAllChild(this.scrollItem.parent, this.scrollItem.name);
        this.scrollItem.active = false;
        let count = this._itemCount;
        if (count > 3) {
            count = 3;
        }
        let itemIndex = this.isFirst(this._currentItem) ? this._currentItem : this.isFinal(this._currentItem) ? this._currentItem - count + 1 : this._currentItem - 1;
        for (let index = 0; index < count; index++) {
            let itemClone = cc.instantiate(this.scrollItem);
            itemClone.name = itemIndex + "Item";
            itemClone.active = true;
            this.node.addChild(itemClone);
            CommonUtil.setLabel(itemClone, "Label", itemIndex);
            if (this._resetCallBack != null) {
                this._resetCallBack(itemClone, itemIndex);
            }
            itemIndex++;
        }
        this._moveCallBack(this._currentItem);

        let posX = 0;
        if (this._itemCount == 2) {
            posX = this.isFirst(this._currentItem) ? this._itemSize.x / 2 : -this._itemSize.x / 2;
        } else if (this._itemCount > 2) {
            posX = this.isFirst(this._currentItem) ? this._itemSize.x : this.isFinal(this._currentItem) ? -this._itemSize.x : 0;
        }
        this.node.setPosition(posX, this.node.position.y);
    }

    touchStart(event: cc.Touch) {
        this.lastUpdateTime = Date.now();

        if (this._isMove) {
            this._canTouchMove = false;
            return;
        }
        if (this._isTouch) {
            return;
        }
        this._canTouchMove = true;
        this._isTouch = true;
        let ray = UIManager.getInstance().getUICamera().getRay(event.getLocation());
        let results = cc.geomUtils.intersect.raycast(cc.director.getScene(), ray);
        for (let i = 0; i < results.length; i++) {
            let node = results[i].node;
            if (node === this.node) {
                this._startPos = event.getLocation();
                this._beginPos = event.getLocation();
                this._currentPos = this.node.position;
                break;
            }
        }
    }

    touchMove(event: cc.Touch) {
        this.lastUpdateTime = Date.now();

        if (!this._canTouchMove) {
            return;
        }
        let ray = UIManager.getInstance().getUICamera().getRay(event.getLocation());
        let results = cc.geomUtils.intersect.raycast(cc.director.getScene(), ray);
        for (let i = 0; i < results.length; i++) {
            let node = results[i].node;
            if (node === this.node) {
                let pos = event.getLocation();
                let moveX = pos.x - this._startPos.x;
                this.node.setPosition(this.node.position.x + moveX, this.node.position.y);
                this._startPos = pos;
                break;
            }
        }
    }

    touchEnd(event: cc.Touch) {
        this.lastUpdateTime = Date.now();

        if (!this._canTouchMove) {
            return;
        }

        this._isTouch = false;

        this.handlePosition(event.getLocation());
    }

    onClickLeftBtn() {
        if (this._isMove) {
            return;
        }
        this._currentPos = this.node.position;
        if (!this.isFirst(this._currentItem)) {
            this._finalIndex--;
            this.move();
        }
    }

    onClickRightBtn() {
        if (this._isMove) {
            return;
        }
        this._currentPos = this.node.position;
        if (!this.isFinal(this._currentItem)) {
            this._finalIndex++;
            this.move();
        }
    }

    move() {
        this._isMove = true;
        let k = this._finalIndex > this._currentItem ? -1 : 1;
        let finalPos = cc.v2(k * this._itemSize.x + this._currentPos.x);
        let self = this;
        this._currentTween = new cc.Tween().target(this.node)
            .to(1, { position: { value: finalPos, easing: "expoOut" } }, null)
            .call(() => {
                if (this._finalIndex > this._currentItem && !this.isFirst(this._currentItem) && !this.isFinal(this._finalIndex)) {
                    let node = self.node.children[1];
                    self.node.removeChild(node);
                    self.node.addChild(node);
                    self.node.setPosition(cc.v2(0, this.node.position.y));
                    node.name = (this._finalIndex + 1) + "Item";
                    if (this._resetCallBack != null) {
                        this._resetCallBack(node, this._finalIndex + 1);
                    }
                } else if (this._finalIndex < this._currentItem && !this.isFirst(this._finalIndex) && !this.isFinal(this._currentItem)) {
                    let node = self.node.children[3];
                    self.node.removeChild(node);
                    self.node.addChild(node);
                    node.setSiblingIndex(1);
                    self.node.setPosition(cc.v2(0, this.node.position.y));
                    node.name = (this._finalIndex - 1) + "Item";
                    if (this._resetCallBack != null) {
                        this._resetCallBack(node, this._finalIndex - 1);
                    }
                }
                this._currentItem = this._finalIndex;
                this._isMove = false;
                this._moveCallBack(this._currentItem);
            })
            .start();
    }

    update(dt: number) {
        if (this._isTouch) {
            if (Date.now() - this.lastUpdateTime > HOLD_MAX_TIME) {
                this._isTouch = false;
                this.handlePosition(this._startPos);
            }
        }
    }

    private handlePosition(position: cc.Vec2): void {
        let ray = UIManager.getInstance().getUICamera().getRay(position);
        let results = cc.geomUtils.intersect.raycast(cc.director.getScene(), ray);
        for (let i = 0; i < results.length; i++) {
            let node = results[i].node;
            if (node === this.node) {
                this._movePos = position.x - this._beginPos.x;
                let canDoMove = Math.abs(this._movePos) > this.offsetX;
                if (canDoMove && this._movePos < 0 && !this.isFinal(this._finalIndex)) {
                    this._finalIndex++;
                    this.move();
                } else if (canDoMove && this._movePos > 0 && !this.isFirst(this._finalIndex)) {
                    this._finalIndex--;
                    this.move();
                } else {
                    this._isMove = true;
                    this._currentTween = new cc.Tween().target(this.node)
                        .to(0.1, { position: { value: cc.v2(this._currentPos), easing: "expoOut" } }, null)
                        .call(() => {
                            this._isMove = false;
                        })
                        .start();
                }
                break;
            }
        }
    }
}
