import ConfigManager from "../../manager/ConfigManager";
import { GameManager } from "../../manager/GameManager";
import UIBase from "../../uiFrame/UIBase";
import CommonUtil from "../../util/CommonUtil";

const { ccclass } = cc._decorator;
@ccclass
export default class ConfirmDialogUI extends UIBase {

    static Create(callback: (res: boolean) => void, createCallback: (view: ConfirmDialogUI) => void) {
        let obj = new ConfirmDialogUI();
        obj.createCallBack = function () {
            obj.attach();
            createCallback(obj);
        };
        obj.createAndAttachRoot("ConfirmDialogUI");
        obj.actionCallback = callback;
        return obj;
    }

    private actionCallback: (res: boolean) => void;

    attach() {
        // ViewManager.BlockInput(true);
        // DragPanel.SetEnable(false);

        let target = this;
        this.setButtonClick("Bg/ButtonLayout/ButtonConfirm",
            () => {
                target.OnClickConfirm();
            },
        );
        this.setButtonClick("Bg/ButtonLayout/ButtonCancel",
            () => {
                target.Destroy();
            });
        this.setButtonClick("Bg/CloseBtn", () => {
            target.Destroy();
        });

        CommonUtil.setLabel(this.root, "Bg/TitleText", ConfigManager.GetText("title_tips"));
        CommonUtil.setLabel(this.root, "Bg/Bg/TitleText", ConfigManager.GetText("title_info"));
    }

    SetText(title: string, content: string, btnConfirmText: string = null, btnCancelText: string = null) {
        this.SetTitle(title != null ? title : ConfigManager.GetText("hint_system"));
        this.SetContent(content);
        this.SetBtnOk(btnConfirmText != null ? btnConfirmText : ConfigManager.GetText("btn_confirm"));
        this.SetBtnCancel(btnCancelText != null ? btnCancelText : ConfigManager.GetText("btn_cancel"));
    }

    SetTextByKey(title: string, content: string, btnConfirmKey: string = null, btnCancelKey: string = null) {
        this.SetTitle(ConfigManager.GetText(title));
        this.SetContent(ConfigManager.GetText(content));
        this.SetBtnOk(ConfigManager.GetText(btnConfirmKey != null ? btnConfirmKey : "btn_confirm"));
        this.SetBtnCancel(ConfigManager.GetText(btnCancelKey != null ? btnCancelKey : "btn_cancel"));
    }

    SetTitle(text: string) {
        CommonUtil.setLabel(this.root, "Bg/TitleText", text);
    }

    SetContent(text: string) {
        CommonUtil.setLabel(this.root, "Bg/TextContent", text);
    }

    SetContentPro(content: string) {
        cc.warn("Unimplemented");
    }
    // CommonUtil.SetTextMeshProUI(this.root, "Bg/TextContentPro", content)
    // return this
    // }

    SetBtnOk(text: string) {
        CommonUtil.setLabel(this.root, "Bg/ButtonLayout/ButtonConfirm/Text", text);
    }

    SetBtnCancel(text: string) {
        CommonUtil.setLabel(this.root, "Bg/ButtonLayout/ButtonCancel/Text", text);
    }

    SetBtnCancelState(canCancel: boolean) {
        CommonUtil.SetActive(this.root, "Bg/ButtonLayout/ButtonCancel", canCancel);
    }

    OnClickConfirm() {
        if (this.actionCallback != null) {
            this.actionCallback(true);
            this.actionCallback = null;
        }
        this.Destroy();
    }

    Destroy() {
        // ViewManager:BlockInput(false)
        // DragPanel.SetEnable(true);

        // GameObject.Destroy(this.root)
        if (this.actionCallback != null) {
            this.actionCallback(false);
            this.actionCallback = null;
        }
    }

}
