import CommonUtil from "../util/CommonUtil";
import UIManager from "./UIManager";

const { ccclass, property } = cc._decorator;

const ANIMATION_TIME: number = 800;

@ccclass
export default class EventLayer extends cc.Component {

    @property(cc.Prefab)
    clickEffect: cc.Prefab = null;

    onLoad(): void {
        let node: any = this.node;
        node.on(cc.Node.EventType.TOUCH_END, this.onTouch, this);
        node._hitTest = this.hitTest.bind(this);

    }
    hitTest(point: cc.Vec2): boolean {

        if (this.clickEffect) {

            let locationInNode = this.node.convertToNodeSpaceAR(point);
            let click = cc.instantiate(this.clickEffect);
            click.position = locationInNode;
            this.node.addChild(click);
            setTimeout(function () {
                click.destroy();
            }.bind(this), ANIMATION_TIME);

            let ray = UIManager.getInstance().getUICamera().getRay(point);
            let results = cc.geomUtils.intersect.raycast(cc.director.getScene(), ray);
            for (let i = 0; i < results.length; i++) {
                let node = results[i].node;
                if (node.hasEventListener(cc.Node.EventType.TOUCH_END)
                    && !CommonUtil.isButton(node)
                    && node != this.node
                    && node.name != "InputMask") {
                    break;
                }
                if (CommonUtil.isButton(node)) {
                    // AudioManager.PlayButton();
                    new cc.Tween().target(node)
                        .to(0.03, { scale: 1.1 }, null)
                        .to(0.01, { scale: 1.0 }, null)
                        .start();
                    break;
                }
            }
        }

        return false;
    }

    onTouch(event: cc.Touch): void {
        cc.log("onTouch: ", event.getLocation());
    }
}
