import ScheduleQueue from "../../manager/ScheduleQueue";
import TextTimer from "../../manager/TextTimer";
import CommonUtil from "../../util/CommonUtil";
import UIManager from "./UIManager";
import UIMaskMgr from "./UIMaskMgr";
import { UIFormsType, UIType } from "./UITypeData";

const { ccclass, property } = cc._decorator;
@ccclass
export default class UIBase extends cc.Component {
    root: cc.Node = null;
    uiName = "";
    uiPath = "";
    currentUIType = new UIType();
    createCallBack = null;
    queue = null;
    private textTimerEnergy = null;

    show() {
        if (this.root == null) {
            return;
        }
        CommonUtil.SetActive(this.root, null, true);
        if (this.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            UIMaskMgr.getInstance().setMaskWindow(this.root, this.currentUIType.UIForms_LucencyType);
        }
    }

    onHide() {

    }

    hide() {
        this.onHide();
        if (this.root == null) {
            return;
        }
        CommonUtil.SetActive(this.root, null, false);
        if (this.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            UIMaskMgr.getInstance().cancelMaskWindow();
        }
    }

    onRedisplay() {

    }

    redisplay() {
        this.onRedisplay();
        if (this.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            this.root.active = true;
            UIMaskMgr.getInstance().setMaskWindow(this.root, this.currentUIType.UIForms_LucencyType);
        }
    }

    createAndAttachRoot(path: string) {
        this.uiName = cc.js.getClassName(this);
        this.uiPath = path;
        let self = this;
        UIManager.getInstance().showUIForm(self);
    }

    addToUIStack(path: string) {
        this.uiName = cc.js.getClassName(this);
        this.uiPath = path;
        let self = this;
        UIManager.getInstance().addNormalUIToStack(self);
    }

    backAView() {
        UIManager.getInstance().backAView();
    }

    closePop() {
        if (!UIManager.getInstance().closeOrReturnUIForms(this.uiName)) {
            this.onDestroy();
        }
    }

    onUpdate(_dt: number) {

    }

    onDestroyRoot() {

    }

    onDestroy() {
        this.onDestroyRoot();
        if (this.root != null) {
            if (this.queue != null) {
                this.unscheduleAllCallbacks();
            }
            this.root.destroy();
            this.root = null;
        }
        if (this.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            UIMaskMgr.getInstance().cancelMaskWindow();
        }

        if (this.textTimerEnergy != null) {
            this.textTimerEnergy.stopTiming();
            this.textTimerEnergy.setCallback(null);
            this.textTimerEnergy = null;
        }
    }

    setButtonClick(path: string, action, ...arg) {
        let btn = CommonUtil.findAChildByType(this.root, path, cc.Button);
        let self = this;
        if (btn == null) {
            cc.warn("no button ", path);
            return;
        }
        btn.node.off("click");
        btn.node.on("click", function () {
            SafeCall(self, action, ...arg);
        });
    }

    setButtonClickByButton(btn: cc.Button, action, ...arg) {
        if (btn == null) {
            return;
        }
        let self = this;
        btn.node.off("click");
        btn.node.on("click", function () {
            SafeCall(self, action, ...arg);
        });
    }

    setToggleClick(path: string, action, ...arg) {
        let tog = CommonUtil.findAChildByType(this.root, path, cc.Toggle);
        let self = this;
        if (tog == null) {
            cc.log("no toggle");
            return;
        }
        tog.node.off("toggle");
        tog.node.on("toggle", function () {
            SafeCall(self, action, tog.isChecked, ...arg);
        });

    }

    setToggleClickByTog(tog: cc.Toggle, action, ...arg) {
        let self = this;
        if (tog == null) {
            cc.log("no toggle");
            return;
        }

        tog.node.off("toggle");
        tog.node.on("toggle", function () {
            SafeCall(self, action, tog.isChecked, ...arg);
        });

    }

    scheduleQueue(func, delay) {
        this.queue = this.root.getComponent(ScheduleQueue);
        if (this.queue == null) {
            this.queue = this.root.addComponent(ScheduleQueue);
        }
        this.queue.add(this, func, delay);
    }

}
