const { ccclass, property } = cc._decorator;

@ccclass
export default class UIButton extends cc.Component {

    private lastX: number;
    private lastY: number;

    private offset: number = 10;

    private onClickFunction: (event) => void = null;

    onLoad() {
        this.node.off(cc.Node.EventType.TOUCH_START);
        this.node.off(cc.Node.EventType.TOUCH_MOVE);
        this.node.off(cc.Node.EventType.TOUCH_END);
        this.node.on(cc.Node.EventType.TOUCH_START, this.onTouchStart, this);
        this.node.on(cc.Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
        this.node.on(cc.Node.EventType.TOUCH_END, this.onTouchEnd, this);
    }

    setButtonClick(action: () => void, target = null) {
        if (action == null) {
            cc.error("Click Action Can not be null");
            return;
        }

        this.onClickFunction = function (event) {
            if (target == null) {
                SafeCall(action, event);
            } else {
                SafeCall(target, action, event);
            }
        };
    }

    private onTouchStart(event) {
        this.lastX = event.getLocation().x;
        this.lastY = event.getLocation().y;
    }

    private onTouchMove(event) {

    }

    private onTouchEnd(event) {
        let x = event.getLocation().x;
        let y = event.getLocation().y;
        if (this.onClickFunction != null
            && Math.abs(x - this.lastX) < this.offset
            && Math.abs(y - this.lastY) < this.offset) {

            this.onClickFunction(event);
        }
    }
}
