
import { PopUpMgr } from "../manager/PopUpMgr";
import ScheduleQueue from "../manager/ScheduleQueue";
import CCDelegate from "../tool/CCDelegate";
import SafeAreaView from "../tool/SafeAreaView";
import CommonUtil from "../util/CommonUtil";
import UIBase from "./UIBase";
import UIMaskMgr from "./UIMaskMgr";
import { UIFormsType } from "./UITypeData";

const { ccclass, property } = cc._decorator;
@ccclass
export default class UIManager extends cc.Component {

    static getInstance(): UIManager {
        if (this._Instance == null) {
            const node = new cc.Node("_UIManager");
            this._Instance = node.addComponent("UIManager");
            cc.game.addPersistRootNode(node);
        }
        return this._Instance;
    }

    private static _Instance = null;
    @property
    _loadingCount = 0;

    @property
    _blockInputCount = 0;

    @property
    _PopLayerMask = null;

    @property
    _waitingPanel: cc.Node = null;

    @property
    _layerNormal: cc.Node = null;
    @property
    _layerPopUp: cc.Node = null;
    @property
    _layerFront: cc.Node = null;
    @property
    _layerWaiting: cc.Node = null;
    @property
    _layerGuide: cc.Node = null;
    @property
    _inputMask: cc.Node = null;

    @property
    _dicAllUIForms = {};
    @property
    _staNormalUIForms: UIBase[] = new Array();
    @property
    _staPopUpUIForms: UIBase[] = new Array();

    @property
    _lastNormalUiBase: UIBase = null;
    queue = null;

    @property
    private _uiCanvas: cc.Node = null;
    @property
    private _uiCamera: cc.Camera = null;

    private _eventLayer: cc.Node = null;

    private _init: boolean = false;

    init(callBack: any) {

        const target = this;
        cc.loader.loadResArray([
            "prefab/uiFrame/UICanvas",
        ], (err: Error, res: any[]) => {
            if (err) {
                cc.error(err);
                return;
            }
            target.setUICanvas(res[0]);
            cc.log("Init UIManager Finished");
            SafeCall(callBack);
        });
    }

    getUICamera() {
        return this._uiCamera;
    }

    getCanvas() {
        return this._uiCanvas;
    }

    setUICanvas(uiCanvasPrefab: cc.Prefab) {
        if (this._uiCanvas == null) {
            this._uiCanvas = cc.instantiate(uiCanvasPrefab);
            cc.game.addPersistRootNode(this._uiCanvas);
            UIMaskMgr.getInstance().setUICanvas(this._uiCanvas);
            this._layerNormal = CommonUtil.findAChild(this._uiCanvas, "Normal");
            this._layerPopUp = CommonUtil.findAChild(this._uiCanvas, "PopUp");
            this._layerFront = CommonUtil.findAChild(this._uiCanvas, "Front");
            this._layerWaiting = CommonUtil.findAChild(this._uiCanvas, "Waiting");
            this._layerGuide = CommonUtil.findAChild(this._uiCanvas, "Guide");
            this._inputMask = CommonUtil.findAChild(this._uiCanvas, "InputMask");
            this._uiCamera = CommonUtil.findAChildByType(this._uiCanvas, "Main Camera", cc.Camera);
            this._eventLayer = CommonUtil.findAChild(this._uiCanvas, "EventLayer");
        }

    }

    showUIForm(uiBase: UIBase) {
        if (uiBase.root != null) {
            uiBase.root.destroy();
        }
        const existUiBase = this._dicAllUIForms[uiBase.uiName];
        if (existUiBase != null) {
            if (uiBase.currentUIType.UIForms_Type == UIFormsType.PopUp
                || uiBase.currentUIType.UIForms_Type == UIFormsType.Front) {
                uiBase.uiName = uiBase.uiName + "&&" + uiBase.uuid;
            }
            if (uiBase.currentUIType.UIForms_Type == UIFormsType.Normal) {
                this.exitUIFormsCache(uiBase.uiName);
            }
        }
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.Normal) {
            this.destroyAllUINormalRoots();
            this.enterUIFormsCache(uiBase);
        }
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            this.pushPopUpForms(uiBase);
        }
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.Front) {
            const self = this;
            this.attachUIRoot(uiBase, function () {
                self._dicAllUIForms[uiBase.uiName] = uiBase;
                uiBase.show();
            });
        }
    }

    addWaitingPanel(waitingPanel) {
        if (this._waitingPanel == null) {
            this._waitingPanel = cc.instantiate(waitingPanel);
            this._layerWaiting.addChild(this._waitingPanel);
            this._layerWaiting.active = false;
        }
    }

    exitUIFormsCache(uiName) {
        if (this._staNormalUIForms.length <= 0) {
            return;
        }
        let existUiBase = this._staNormalUIForms.find(function (v) {
            return v.uiName == uiName;
        });
        remove(this._staNormalUIForms, existUiBase);
        existUiBase = this._dicAllUIForms[uiName];
        if (existUiBase != null) {
            delete this._dicAllUIForms[uiName];
            existUiBase.onDestroy();
        }
    }

    destroyAllUINormalRoots() {
        this.closePopUIByLinkUIName("");
        for (const uiBase of this._staNormalUIForms) {
            if (uiBase.currentUIType.IsLobbyView) {
                uiBase.hide();
            } else {
                uiBase.onDestroy();
            }
        }
    }

    enterUIFormsCache(uiBase: UIBase) {
        if (uiBase.currentUIType.IsLobbyView) {
            this.clearNormalCache();
        }
        this.removeFromStack(uiBase);
        const self = this;
        this.attachUIRoot(uiBase, function () {
            uiBase.show();
            self.addNormalUIToStack(uiBase);
            self.setLinkPopUpFormsActive(uiBase);
        });
    }

    clearNormalCache() {
        for (const uiBase of this._staNormalUIForms) {
            delete this._dicAllUIForms[uiBase.uiName];
        }
        this._staNormalUIForms = new Array();
        for (const key of Object.keys(this._dicAllUIForms)) {
            delete this._dicAllUIForms[key];
        }
    }

    removeFromStack(uiBase: UIBase) {
        if (this._staNormalUIForms.length > 0
            && this._staNormalUIForms[this._staNormalUIForms.length - 1].uiName == uiBase.uiName) {
            this._staNormalUIForms.pop();
        }
    }

    attachUIRoot(uiBase: UIBase, callBack = null) {
        this.CloseLastNormalUI(uiBase);
        UIManager.getInstance().blockInput(true);
        const self = this;
        CommonUtil.loadPrefab(uiBase.uiPath, function (err, prefab) {
            UIManager.getInstance().blockInput(false);
            if (err != null) {
                cc.error(err);
            }
            uiBase.root = cc.instantiate(prefab);
            if (uiBase.root == null) {
                cc.log("root 为空");
                return;
            }
            if (uiBase.currentUIType.UIForms_Type == UIFormsType.Normal) {
                self._layerNormal.addChild(uiBase.root);
            }
            if (uiBase.currentUIType.UIForms_Type == UIFormsType.PopUp) {
                self._layerPopUp.addChild(uiBase.root);
            }
            if (uiBase.currentUIType.UIForms_Type == UIFormsType.Front) {
                self._layerFront.addChild(uiBase.root);
            }
            let ccDelegate = uiBase.root.addComponent(CCDelegate);
            ccDelegate.registerOnUpdate(function (dt) {
                SafeCall(uiBase, uiBase.onUpdate, dt);
            });
            SafeCall(callBack);
            SafeCall(uiBase, uiBase.createCallBack);
        });
    }

    CloseLastNormalUI(curUIBase: UIBase) {
        if (curUIBase.currentUIType.UIForms_Type != UIFormsType.Normal) {
            return;
        }

        if (this._lastNormalUiBase == null) {
            return;
        }

        if (this._dicAllUIForms[this._lastNormalUiBase.uiName] != null) {
            delete this._dicAllUIForms[this._lastNormalUiBase.uiName];
        }

        this._lastNormalUiBase.onDestroy();
        this._lastNormalUiBase = null;
    }

    addNormalUIToStack(uiBase: UIBase) {
        if (uiBase.currentUIType.UIForms_Type != UIFormsType.Normal) {
            return;
        }
        if (this._dicAllUIForms[uiBase.uiName] != null) {
            this.removeFromStack(uiBase);
            delete this._dicAllUIForms[uiBase.uiName];
        }
        this._dicAllUIForms[uiBase.uiName] = uiBase;
        this._staNormalUIForms.push(uiBase);
    }

    pushPopUpForms(uiBase: UIBase) {
        const self = this;
        this.attachUIRoot(uiBase, function () {
            self._dicAllUIForms[uiBase.uiName] = uiBase;

            self.hideShowingPopView();
            self._staPopUpUIForms.push(uiBase);
            uiBase.show();
            new cc.Tween().target(uiBase.root)
                .to(0.1, { scale: { value: 1.1, easing: "expoOut" } }, null)
                .to(0.05, { scale: { value: 1, easing: "expoOut" } }, null)
                .start();
        });

    }

    hideShowingPopView() {
        if (this._staPopUpUIForms.length > 0) {
            const existUiBase = this._staPopUpUIForms[this._staPopUpUIForms.length - 1];
            if (!existUiBase.currentUIType.NoHidePopUp) {
                existUiBase.hide();
            }
        }
    }

    popPopUpForms(uiBase: UIBase) {
        if (this._staPopUpUIForms.length < 1) {
            return;
        }
        let topUIForms = this._staPopUpUIForms.find(function (v) { return v.uiName == uiBase.uiName; });
        if (topUIForms != null) {
            remove(this._staPopUpUIForms, topUIForms);
        } else {
            topUIForms = this._staPopUpUIForms.pop();
        }
        delete this._dicAllUIForms[uiBase.uiName];
        topUIForms.onDestroy();
        let doPop = false;
        if (topUIForms.currentUIType.PopType != "") {
            doPop = PopUpMgr.doPopAction(topUIForms.currentUIType.PopType);
        }
        if (!doPop) {
            if (this._staPopUpUIForms.length >= 1) {
                this.redisplayPopupForms();
            }
        }
    }

    setLinkPopUpFormsActive(uiBase) {
        if (this._staPopUpUIForms.length < 1) {
            return;
        }
        for (const iterator of this._staPopUpUIForms) {
            if (iterator.currentUIType.LinkUIName == uiBase.uiName) {
                iterator.show();
                return;
            }
        }
    }

    redisplayPopupForms() {
        for (let index = this._staPopUpUIForms.length - 1; index >= 0; index--) {
            let uiBase = this._staPopUpUIForms[index];
            if (uiBase.currentUIType.LinkUIName != "" && uiBase.currentUIType.LinkUIName != this.getCurrentNormalView().uiName) {
                continue;
            }
            uiBase.root.setSiblingIndex(this._layerPopUp.childrenCount);
            uiBase.redisplay();
            return true;
        }
        return false;
    }

    backAView() {
        if (this._staNormalUIForms.length <= 1) {
            const uiBase = this._staNormalUIForms.pop();
            if (uiBase.currentUIType.IsLobbyView) {
                uiBase.redisplay();
            } else {
                uiBase.onDestroy();
            }
            return;
        }
        this._lastNormalUiBase = this._staNormalUIForms.pop();
        this.closePopUIByLinkUIName(this._lastNormalUiBase.uiName);
        const nowBase = this._staNormalUIForms[this._staNormalUIForms.length - 1];
        nowBase.redisplay();
        if (nowBase.currentUIType.IsLobbyView) {
            this.CloseLastNormalUI(nowBase);
        }
    }

    closePopUIByLinkUIName(name) {
        for (let index = this._staPopUpUIForms.length - 1; index >= 0; index--) {
            let uiBase = this._staPopUpUIForms[index];
            if (uiBase.currentUIType.LinkUIName == name || uiBase.currentUIType.LinkUIName == "") {
                uiBase.onDestroy();
                remove(this._staPopUpUIForms, uiBase);
                delete this._dicAllUIForms[uiBase.uiName];
            } else {
                uiBase.hide();
            }
        }
    }

    closeOrReturnUIForms(uiName) {
        let uiBase;
        if (this._dicAllUIForms[uiName] == null) {
            cc.warn("没有找到此UI界面 name:" + uiName);
            return false;
        }
        uiBase = this._dicAllUIForms[uiName];
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.Normal) {
            this.exitUIFormsCache(uiName);
        }
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.PopUp) {
            this.popPopUpForms(uiBase);
        }
        if (uiBase.currentUIType.UIForms_Type == UIFormsType.Front) {
            const uiForms = this._dicAllUIForms[uiBase.uiName];
            uiForms.onDestroy();
            delete this._dicAllUIForms[uiBase.uiName];
        }
        return true;
    }

    showLoading() {
        this._loadingCount++;
        this._layerWaiting.active = this._loadingCount > 0;
        this.blockInput(true);
    }

    hideLoading() {
        this._loadingCount--;
        this._layerWaiting.active = this._loadingCount > 0;
        this.blockInput(false);
    }

    blockInput(block) {
        if (block) {
            this._blockInputCount++;
        } else {
            this._blockInputCount--;
        }
        this._inputMask.active = this._blockInputCount > 0;
    }

    asyncCallBlockInput(func, delay) {
        this.blockInput(true);
        this.queue = this._uiCanvas.getComponent(ScheduleQueue);
        if (this.queue == null) {
            this.queue = this._uiCanvas.addComponent(ScheduleQueue);
        }
        this.queue.add(this, function () {
            this.blockInput(false);
            SafeCall(func);
        }, delay);
    }

    syncViewToNormalLayer(layer: cc.Node) {
        cc.log("SyncViewToNormalLayer Size w: %s, h: %s, s: %s", this._layerNormal.width, this._layerNormal.height, this._layerNormal.scale);
        layer.width = this._layerNormal.width;
        layer.height = this._layerNormal.height;
        layer.scale = this._layerNormal.scale;
        layer.position = this._layerNormal.position;
    }

    getCurrentNormalView() {
        if (this._staNormalUIForms.length < 1) {
            return null;
        }
        return this._staNormalUIForms[this._staNormalUIForms.length - 1];
    }

    getCurrentShowingUIForms() {
        let showingView = new Array();
        for (const key of Object.keys(this._dicAllUIForms)) {
            const view = this._dicAllUIForms[key];
            if (view.root != null && view.root.active) {
                showingView.push(view);
            }
        }
        return showingView;
    }

    refreshCurrentViewResource() {
        for (const view of this.getCurrentShowingUIForms()) {
            view.showUserResource();
        }
    }

    getScale() {
        let safeArea = this._uiCanvas.getComponent(SafeAreaView);
        if (safeArea != null) {
            return safeArea.getScale();
        }
    }

    isCurrentViewPlayUI() {
        let current = this.getCurrentNormalView();
        if (current == null) {
            return false;
        }
        return current.uiName == "PlayUI";
    }

    isCurrentViewIsLobbyView() {
        let current = this.getCurrentNormalView();
        if (current == null) {
            return false;
        }
        return current.currentUIType.IsLobbyView;
    }

    backToLobbyView(needHide = false) {
        let lobbyUI = null;
        for (const iterator of Object.keys(this._dicAllUIForms)) {
            const uiBase = this._dicAllUIForms[iterator];
            if (uiBase.currentUIType.IsLobbyView) {
                lobbyUI = uiBase;
            } else if (uiBase.root != null) {
                uiBase.onDestroy();
            }
            delete this._dicAllUIForms[iterator];
        }
        this._staNormalUIForms = [];
        this._staPopUpUIForms = [];
        if (lobbyUI != null) {
            this._dicAllUIForms[lobbyUI.uiName] = lobbyUI;
            this._staNormalUIForms.push((lobbyUI));
            if (!needHide) {
                lobbyUI.redisplay();
            }
        }
    }
}
