import CommonUtil from "./CommonUtil";

export default class DragonBonesUtil {

    /// 加载龙骨预设
    /// <param name="path">Resources后的路径名</param>
    static LoadDragonBones(path: string, callback: (res: cc.Asset) => void = null): void {
        // Object a = HotfixManager.Instance.LoadResource(path + "_Data" , typeof(Object));
        // UnityDragonBonesData b = (UnityDragonBonesData)a;
        // UnityFactory.factory.LoadData(b);
        cc.loader.loadRes(path, dragonBones.DragonBonesAsset, (err: Error, res) => {
            if (err) {
                cc.error(err);
            }
            if (null != callback) {
                callback(res);
            }
        });
    }

    static GetDBComponent(node: cc.Node): dragonBones.ArmatureDisplay {
        if (null == node) {
            cc.warn("node cannot be null!");
            return null;
        }
        return node.getComponent(dragonBones.ArmatureDisplay);
    }

    static PlayAnime(armatureNode: cc.Node, action: string, playTimes: number = -1): void {
        let armature = armatureNode.getComponent(dragonBones.ArmatureDisplay);
        armature.playAnimation(action, playTimes);
    }

    static SetChildArmature(armatureComponent: cc.Node, slotName: string, armature: any): void {
        // armatureComponent.GetSlot(slotName).childArmature = armature;
    }

    static GetSkinData(skinName: string): any {
        let factory = dragonBones.CCFactory.getInstance();
        let armatureData: dragonBones.ArmatureData = factory.getArmatureData(skinName);
        if (armatureData == null) {
            cc.warn("No armatureData for key: ", skinName);
        }
        return armatureData.defaultSkin;
    }

    static ReplaceSkin(armatureNode: cc.Node, skin: dragonBones.SkinData, isOverride: boolean = false, exclude?: string[] | null): boolean {
        let armature = armatureNode.getComponent(dragonBones.ArmatureDisplay).armature();
        return dragonBones.CCFactory.getInstance().replaceSkin(armature, skin, isOverride, exclude);
    }

    static SetAnimeTimeScale(armatureNode: cc.Node, value: number): void {
        let armature = armatureNode.getComponent(dragonBones.ArmatureDisplay).armature();
        armature.animation.timeScale = value;
    }

    static SetMaterialColor(armatureNode: cc.Node, color: cc.Color): void {
        // TODO
    }

    static GetSlot(armatureNode: cc.Node, slotName: string): any {
        let armature = armatureNode.getComponent(dragonBones.ArmatureDisplay).armature();
        return armature.getSlot(slotName);
    }

    static BuildArmature(armatureName: string, dragonBonesName?: string, skinName?: string, textureAtlasName?: string): dragonBones.Armature | null {
        return dragonBones.CCFactory.getInstance().buildArmature(armatureName, dragonBonesName, skinName, textureAtlasName);
    }

    static SetSlotDisplay(slot: any, dragonBonesName: string, armatureName: string, slotName: string, displayName: string, displayIndex: number = -1): void {
        dragonBones.CCFactory.getInstance().replaceSlotDisplay(dragonBonesName, armatureName, slotName, displayName, slot, displayIndex);
    }

    /**
     * 淡入播放指定的动画。
     * @param animationName - 动画数据名称。
     * @param fadeInTime - 淡入时间。 [-1: 使用动画数据默认值, [0~N]: 淡入时间 (以秒为单位)] （默认: -1）
     * @param playTimes - 播放次数。 [-1: 使用动画数据默认值, 0: 无限循环播放, [1~N]: 循环播放 N 次] （默认: -1）
     * @param layer - 混合图层，图层高的动画状态会优先获取混合权重，当混合权重分配总和超过 1.0 时，剩余的动画状态将不能再获得权重分配。 （默认: 0）
     * @param group - 混合组名称，该属性通常用来指定多个动画状态混合时的相互替换关系。 （默认: null）
     * @param fadeOutMode - 淡出模式，该属性通常用来指定多个动画状态混合时的相互替换模式。 （默认: AnimationFadeOutMode.SameLayerAndGroup）
     * @returns 播放的动画状态。
     */
    static FadeIn(armatureNode: cc.Node, animationName: string, fadeInTime = -1.0, playTimes = -1, layer = 0, group: string = null, fadeOutMode = dragonBones.AnimationFadeOutMode.SameLayerAndGroup): dragonBones.AnimationState {
        let armature = armatureNode.getComponent(dragonBones.ArmatureDisplay).armature();
        let animation: dragonBones.Animation = armature.animation;
        return animation.fadeIn(animationName, fadeInTime, playTimes, layer, group, fadeOutMode);
    }
}
