const fs = require("fire-fs");
const fse = require('fs-extra');
const path = require("fire-path");

var self = {
    isMeta: function (filename) {
        return path.extname(filename) == ".meta";
    },

    getUUID: function (fileName) {
        let jsonNode = JSON.parse(fs.readFileSync(fileName));
        return jsonNode.uuid;
    },

    startWith(src, str) {
        return src.indexOf(str) == 0;
    },

    getBaseName(fileName) {
        return path.basename(fileName, path.extname(fileName));
    },

    getOSSBucketPath(fPath) {
        let fileContent = fs.readFileSync(fPath, 'utf-8');
        let m = fileContent.match(/"(.*:\/\/[^/].*)\/([^/].*)"/ig);
        if (m.length > 0) {
            return RegExp.$2;
        }

        return "";
    },


    copyDir(srcPath, tarPath, filter = []) {
        fse.ensureDirSync(tarPath);
        fs.readdir(srcPath, function (err, files) {
            if (err === null) {
                files.forEach(function (filename) {
                    let fileDir = path.join(srcPath, filename);
                    let filterFlag = filter.some(item => item === filename);
                    if (!filterFlag) {
                        fs.stat(fileDir, function (errs, stats) {
                            let isFile = stats.isFile();
                            if (isFile) {                                    // 复制文件
                                const destPath = path.join(tarPath, filename);
                                fse.copySync(fileDir, destPath, { overwrite: true });
                            } else {                                        // 创建文件夹
                                let tFileDir = path.join(tarPath, filename);
                                fse.ensureDirSync(tFileDir);
                                self.copyDir(fileDir, tFileDir, filter);                 // 递归
                            }
                        });
                    }
                });
            } else {
                if (err) console.error(err);
            }
        });
    },

    walkFiles: function (dir, process, processParam) {

        // Editor.log('WalkFiles :' + dir);
        let fPaths = fs.readdirSync(dir);
        for (const file of fPaths) {
            let fullPath = path.join(dir, file);
            if (fs.statSync(fullPath).isDirectory()) {
                self.walkFiles(fullPath, process, processParam);
            } else {
                process(fullPath, processParam);
            }
        }
    },

    promiseAll: function (taskList, step, callback) {
        let tmpTaskList = [];
        for (let i = 0; i < step; i++) {
            if (taskList.length <= 0) {
                break;
            }
            let p = taskList.shift();
            tmpTaskList.push(p());
        }
        if (tmpTaskList.length == 0) {
            // Editor.info("Upload Task resolve");
            callback();
            return;
        }
        Promise.all(tmpTaskList)
            .then(() => {
                self.promiseAll(taskList, step, callback);
            }).catch(() => {
                self.promiseAll(taskList, step, callback);
            });
    },
};

module.exports = self;