// panel/index.js, this filename needs to match the one registered in package.json
/* global Editor */
const fs = require('fire-fs');
const path = require("fire-path");
// const utils = Editor.require('packages://build_index/buildUtils.js');

let self = {
    // css style for panel
    style: fs.readFileSync(Editor.url("packages://build_index/panel/index.css"), 'utf-8'),

    // html template for panel
    template: fs.readFileSync(Editor.url("packages://build_index/panel/index.html"), 'utf-8'),

    // element and variable binding
    $: {
        btnCreateSpriteIndex: '#btnCreateSpriteIndex',
        btnCreatePrefabIndex: '#btnCreatePrefabIndex',
        btnProcessWXProject: '#btnProcessWXProject',
        btnUploadWXProject: '#btnUploadWXProject',
        btnReloadPlugins: '#btnReloadPlugins',
        btnRunWX: '#btnRunWX',
        lbState: '#lbState',
    },

    // method executed when template and styles are successfully loaded and initialized
    ready() {

        this.$btnCreateSpriteIndex.addEventListener('confirm', () => {
            Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', "CreateSpriteIndex");
            Editor.Ipc.sendToMain('build_index:createSpriteIndex');
        });
        this.$btnCreatePrefabIndex.addEventListener('confirm', () => {
            Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', "CreatePrefabIndex");
            Editor.Ipc.sendToMain('build_index:createPrefabIndex');
        });
        this.$btnProcessWXProject.addEventListener('confirm', () => {
            Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', "ProcessWXProject");
            Editor.Ipc.sendToMain('build_index:processWXProject', function (error, answer) {
                Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', answer);
            });
        });
        this.$btnUploadWXProject.addEventListener('confirm', () => {
            Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', "UploadWXProject");
            Editor.Ipc.sendToMain('build_index:uploadWXProject', function (error, answer) {
                Editor.Ipc.sendToPanel('build_index', 'build_index:setLbText', answer);
            });
        });

        this.$btnReloadPlugins.addEventListener('confirm', () => {
            Editor.Ipc.sendToMain('plugin_debug:reloadPlugin');
        });

        this.$btnRunWX.addEventListener('confirm', () => {
            let editorSettingPath = path.join(process.env.USERPROFILE, ".CocosCreator", "settings.json");
            let jsonNode = JSON.parse(fs.readFileSync(editorSettingPath));
            let execPath = path.join(jsonNode["wechatgame-app-path"], 'cli.bat');
            Editor.Utils.run(
                execPath,
                "-o",
                path.join(Editor.Project.path, "build", "wechatgame")
            );
        });
    },

    // register your ipc messages here
    messages: {
        'build_index:setLbText': function (event, text) {
            this.$lbState.innerText = text;
        },

    },

};


Editor.Panel.extend(self);